==> cf-report-image-toolbox-1.0.14.tgz: package/src/api/registries.ts <==
import _ from 'lodash'
import { STS } from '@aws-sdk/client-sts'
import { parseQualifiedNameOptimized, parseFamiliarName } from '@codefresh-io/docker-reference'
import { registries } from 'nodegistry'

import { CF_NOT_EXIST, CommonInstructions } from './types'
import { ReportImageInputs } from '../report-image-info/types'
import fs, { promises as fsPromise } from 'fs'
const { GcrRegistry, EcrRegistry, DockerhubRegistry, StandardRegistry } = registries

export class RegistryClient {
    private readonly inputs: ReportImageInputs

    constructor(inputs) {
        this.inputs = inputs
    }

    checkNotEmpty(testVar) {
        return (testVar && testVar !== CF_NOT_EXIST)
    }

    parseImageName(imageName) {
        return parseFamiliarName(imageName, parseQualifiedNameOptimized)
    }

    getCredentialsFromDockerConfig(image) {
        const dockerConfig = JSON.parse(fs.readFileSync(this.inputs.dockerConfigPath).toString())
        const imageData = this.parseImageName(image)
        const auths = _.get(dockerConfig, 'auths', {})
        const domainKey = _.findKey(auths, (auth, domain) => {
            if (domain.includes(imageData.domain)) {
                return true
            }
        })
        if (!domainKey) {
            throw new Error('Domain is not provided.')
        }
        const authInfo = auths[domainKey]
        const auth = this.decodeBase64(authInfo.auth)
        const [ username, password ] = auth.split(':')
        if (domainKey.includes('docker.io')) {
            return new DockerhubRegistry({
                username,
                password,
            })
        } else if (domainKey.includes('azurecr.io')) {
            throw new Error('Azure Container Registry using docker config json is not supported.')
        } else if (domainKey.includes('gcr.io')) {
            throw new Error('Google container registry using docker config json is not supported.')
        }
        return new StandardRegistry({
            request: {
                host: domainKey
            },
            credentials: {
                username,
                password,
            },
            ignoreRedirects: true
        })
    }

    async createECRUsingSTS(role, region) {
        console.log(`Retrieving credentials for ECR ${region} using STS token`)
        try {
            const sts = new STS({ region })
            sts.middlewareStack.add((next) => (args: any) => {
                  console.log('\n -- printed from inside STS middleware -- ')
                  console.log('hostname:', args.request.hostname)
                  console.log('input:', args.input, '\n')
                  return next(args)
              },
              {
                  step: 'build',
              })
            const timestamp = Date.now()
            /* eslint-disable @typescript-eslint/naming-convention */
            const params = {
                RoleArn: role,
                RoleSessionName: `be-descriptibe-here-${timestamp}`
            }
            /* eslint-enable @typescript-eslint/naming-convention */
            const data = await sts.assumeRole(params)
            return new EcrRegistry({
                promise: Promise,
                credentials: {
                    accessKeyId: data.Credentials?.AccessKeyId,
                    secretAccessKey: data.Credentials?.SecretAccessKey,
                    sessionToken: data.Credentials?.SessionToken,
                    region: region,
                },
            })
        } catch (error) {
            console.error(error.message)
            throw error
        }
    }

    async createRegistryClientByImage(image) {
        const imageData = this.parseImageName(image)
        if (imageData.domain.includes('docker.io')) {
            if (this.checkNotEmpty(this.inputs.dockerhub?.username)
              && this.checkNotEmpty(this.inputs.dockerhub?.password)) {

                return new DockerhubRegistry(this.inputs.dockerhub)
            }
            throw new Error('Registry credentials for DOCKER not set. Add following registry parameters in your workflow to continue:\n - DOCKERHUB_USERNAME\n - DOCKERHUB_PASSWORD\n')
        } else if (imageData.domain.includes('gcr.io')) {
            if (this.inputs.gcr?.keyFilePath) {
                return new GcrRegistry({
                    keyfile: fs.readFileSync(this.inputs.gcr.keyFilePath),
                    request: { host: 'gcr.io' }
                })
            }
            throw new Error('Registry credentials for GCR not set. Add following registry parameters in your workflow to continue:\n - GCR_KEY_FILE_PATH\n')
        } else if (imageData.domain.includes('azurecr.io')) {
            throw new Error('Azure Container Registry using docker config json is not supported.')
        } else if (imageData.domain.includes('ecr')) {
            if (this.checkNotEmpty(this.inputs.aws?.role)
              && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
                return this.createECRUsingSTS(this.inputs.aws?.role, this.inputs.aws?.credentials?.region)
            } else if (this.checkNotEmpty(this.inputs.aws?.credentials?.accessKeyId)
              && this.checkNotEmpty(this.inputs.aws?.credentials?.secretAccessKey)
              && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
                return new EcrRegistry({
                    promise: Promise,
                    credentials: this.inputs.aws?.credentials,
                })
            }
            throw new Error('Registry credentials for ECR not set. Add following registry parameters in your workflow to continue:\n - AWS_ACCESS_KEY\n - AWS_SECRET_KEY\n - AWS_REGION\n')
        }
        if (this.checkNotEmpty(this.inputs.generic?.credentials?.username)
          && this.checkNotEmpty(this.inputs.generic?.credentials?.password)
          && this.checkNotEmpty(this.inputs.generic?.request?.host)) {

            _.set(this.inputs, 'generic.ignoreRedirects', true)
            return new StandardRegistry(this.inputs.generic)
        }
        if (this.inputs.dockerConfigPath) {
            return this.getCredentialsFromDockerConfig(image)
        }
        throw new Error('Registry credentials is required parameter. Add one from following registry parameters in your workflow to continue:\n - Docker credentials: DOCKERHUB_USERNAME, DOCKERHUB_PASSWORD\n - GCR credentials: GCR_KEY_FILE_PATH\n - AWS registry credentials: AWS_ACCESS_KEY, AWS_SECRET_KEY, AWS_REGION\n - Standard registry credentials: REGISTRY_USERNAME, REGISTRY_PASSWORD, REGISTRY_DOMAIN\n')
    }

    async createRegistryClient(image) {
        if (this.inputs.dockerConfigPath) {
            return this.getCredentialsFromDockerConfig(image)
        }

        // Clean this up when have time
        if (this.checkNotEmpty(this.inputs.dockerhub?.username)
          && this.checkNotEmpty(this.inputs.dockerhub?.password)) {

            return new DockerhubRegistry(this.inputs.dockerhub)
        }

        if (this.checkNotEmpty(this.inputs.generic?.credentials?.username)
          && this.checkNotEmpty(this.inputs.generic?.credentials?.password)
          && this.checkNotEmpty(this.inputs.generic?.request?.host)) {

            _.set(this.inputs, 'generic.ignoreRedirects', true)
            return new StandardRegistry(this.inputs.generic)
        }

        if (this.checkNotEmpty(this.inputs.aws?.role)
          && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
            return this.createECRUsingSTS(this.inputs.aws?.role, this.inputs.aws?.credentials?.region)
        }

        if (this.inputs.gcr?.keyFilePath) {
            return new GcrRegistry({
                keyfile: fs.readFileSync(this.inputs.gcr.keyFilePath),
                request: { host: this.inputs.gcr.host || 'gcr.io' }
            })
        }
        if (this.checkNotEmpty(this.inputs.gcr?.json)) {
            return new GcrRegistry({
                keyfile: this.inputs.gcr?.json,
                request: { host: this.inputs.gcr?.host || 'gcr.io' }
            })
        }
        if (this.checkNotEmpty(this.inputs.aws?.credentials?.accessKeyId)
          && this.checkNotEmpty(this.inputs.aws?.credentials?.secretAccessKey)
          && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
            return new EcrRegistry({
                promise: Promise,
                credentials: this.inputs.aws?.credentials,
            })
        }
        throw new Error('Registry credentials is required parameter. Add one from following registry parameters in your workflow to continue:\n - Docker credentials: DOCKERHUB_USERNAME, DOCKERHUB_PASSWORD\n - GCR credentials: GCR_KEY_FILE_PATH\n - AWS registry credentials: AWS_ACCESS_KEY, AWS_SECRET_KEY, AWS_REGION\n - Standard registry credentials: REGISTRY_USERNAME, REGISTRY_PASSWORD, REGISTRY_DOMAIN')
    }

    async getRegistryClient(image) {
        if (this.inputs.retrieveCredentialsByDomain) {
            return this.createRegistryClientByImage(image)
        }
        return this.createRegistryClient(image)
    }

    /**
     * @param history {[*]}
     * @returns {[{size, created: (string|Boolean), instruction: string, args: string}]}
     */
    getLayersFromHistory(history, layers) {

        let shell = '/bin/sh -c'
        const instructionPrefix = '#(nop)'

        const layersSizes = _.map(layers, l => l.size)

        history = history.reverse().map((row) => {
            row = row || {}

            const CreatedBy = _.get(row, 'created_by', '')
            let instruction, args

            if (!CreatedBy.includes(instructionPrefix)) {
                instruction = CommonInstructions.run
                args        = CreatedBy.replace(shell, '').trim()
            } else {
                const cmd = CreatedBy.slice(CreatedBy.indexOf(instructionPrefix) + instructionPrefix.length).trim()
                const nextSpacePosition = cmd.indexOf(' ')

                instruction = cmd.slice(0, Math.max(0, nextSpacePosition)).trim()
                args        = cmd.slice(nextSpacePosition).trim()

                if (instruction === CommonInstructions.shell) {
                    shell = (args[0] === '[' && args[args.length - 1] === ']') ? (args
                      .slice(1, -1)
                      .replace(/["']/g, '')
                      .split(',')
                      .map(part => {return part.trim()})
                      .join(' ')) : args
                }
            }

            return {
                created: row.created,
                size: row.empty_layer ? 0 : layersSizes.pop(),
                instruction,
                args
            }
        }).reverse()

        return history
    }

    async buildDockerfile() {
        try {
            const { dockerfileContent, dockerfilePath } = this.inputs
            if (dockerfileContent) {
                return Buffer.from(dockerfileContent, 'base64').toString()
            }
            const file = await fsPromise.readFile(dockerfilePath ? dockerfilePath : '/tmp/Dockerfile') // use path or artifact
            return file.toString()
        } catch (error) {
            console.error(`Can't get Dockerfile. ${error.message}`)
            return ''
        }
    }

    getRegistryImageSize(manifest) {
        return manifest.config.size + _.reduce(manifest.layers, (sum, layer) => {
            return sum + layer.size
        }, 0)
    }

    private decodeBase64 = (str) => Buffer.from(str, 'base64').toString()
}

==> cf-report-image-toolbox-1.0.17.tgz: package/src/api/registries.ts <==
import _ from 'lodash'
import { STS } from '@aws-sdk/client-sts'
import { parseQualifiedNameOptimized, parseFamiliarName } from '@codefresh-io/docker-reference'
import { registries } from 'nodegistry'

import { CF_NOT_EXIST, CommonInstructions } from './types'
import { ReportImageInputs } from '../report-image-info/types'
import fs, { promises as fsPromise } from 'fs'
const { GcrRegistry, EcrRegistry, DockerhubRegistry, StandardRegistry } = registries

export class RegistryClient {
    private readonly inputs: ReportImageInputs

    constructor(inputs) {
        this.inputs = inputs
    }

    checkNotEmpty(testVar) {
        return (testVar && testVar !== CF_NOT_EXIST)
    }

    parseImageName(imageName) {
        return parseFamiliarName(imageName, parseQualifiedNameOptimized)
    }

    getCredentialsFromDockerConfig(image) {
        const dockerConfig = JSON.parse(fs.readFileSync(this.inputs.dockerConfigPath).toString())
        const imageData = this.parseImageName(image)
        const auths = _.get(dockerConfig, 'auths', {})
        const domainKey = _.findKey(auths, (auth, domain) => {
            if (domain.includes(imageData.domain)) {
                return true
            }
        })
        if (!domainKey) {
            throw new Error('Domain is not provided.')
        }
        const authInfo = auths[domainKey]
        const auth = this.decodeBase64(authInfo.auth)
        const [ username, password ] = auth.split(':')
        if (domainKey.includes('docker.io')) {
            return new DockerhubRegistry({
                username,
                password,
            })
        } else if (domainKey.includes('azurecr.io')) {
            throw new Error('Azure Container Registry using docker config json is not supported.')
        } else if (domainKey.includes('gcr.io')) {
            throw new Error('Google container registry using docker config json is not supported.')
        }
        return new StandardRegistry({
            request: {
                host: domainKey
            },
            credentials: {
                username,
                password,
            },
            ignoreRedirects: true
        })
    }

    async createECRUsingSTS(role, region) {
        console.log(`Retrieving credentials for ECR ${region} using STS token`)
        try {
            const sts = new STS({ region })
            sts.middlewareStack.add((next) => (args: any) => {
                  console.log('\n -- printed from inside STS middleware -- ')
                  console.log('hostname:', args.request.hostname)
                  console.log('input:', args.input, '\n')
                  return next(args)
              },
              {
                  step: 'build',
              })
            const timestamp = Date.now()
            /* eslint-disable @typescript-eslint/naming-convention */
            const params = {
                RoleArn: role,
                RoleSessionName: `be-descriptibe-here-${timestamp}`
            }
            /* eslint-enable @typescript-eslint/naming-convention */
            const data = await sts.assumeRole(params)
            return new EcrRegistry({
                promise: Promise,
                credentials: {
                    accessKeyId: data.Credentials?.AccessKeyId,
                    secretAccessKey: data.Credentials?.SecretAccessKey,
                    sessionToken: data.Credentials?.SessionToken,
                    region: region,
                },
            })
        } catch (error) {
            console.error(error.message)
            throw error
        }
    }

    async createRegistryClientByImage(image) {
        const imageData = this.parseImageName(image)
        if (imageData.domain.includes('docker.io')) {
            if (this.checkNotEmpty(this.inputs.dockerhub?.username)
              && this.checkNotEmpty(this.inputs.dockerhub?.password)) {

                return new DockerhubRegistry(this.inputs.dockerhub)
            }
            throw new Error('Registry credentials for DOCKER not set. Add following registry parameters in your workflow to continue:\n - DOCKERHUB_USERNAME\n - DOCKERHUB_PASSWORD\n')
        } else if (imageData.domain.includes('gcr.io') || this.inputs.gcr?.host && imageData.domain.includes(this.inputs.gcr.host)) {
            if (this.inputs.gcr?.keyFilePath) {
                return new GcrRegistry({
                    keyfile: fs.readFileSync(this.inputs.gcr.keyFilePath),
                    request: { host: this.inputs.gcr.host || 'gcr.io'  }
                })
            }
            if (this.checkNotEmpty(this.inputs.gcr?.json)) {
                return new GcrRegistry({
                    keyfile: this.inputs.gcr?.json,
                    request: { host: this.inputs.gcr?.host || 'gcr.io' }
                })
            }
            throw new Error('Registry credentials for GCR not set. Add following registry parameters in your workflow to continue:\n - GCR_KEY_FILE_PATH\n')
        } else if (imageData.domain.includes('azurecr.io')) {
            throw new Error('Azure Container Registry using docker config json is not supported.')
        } else if (imageData.domain.includes('ecr')) {
            if (this.checkNotEmpty(this.inputs.aws?.role)
              && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
                return this.createECRUsingSTS(this.inputs.aws?.role, this.inputs.aws?.credentials?.region)
            } else if (this.checkNotEmpty(this.inputs.aws?.credentials?.accessKeyId)
              && this.checkNotEmpty(this.inputs.aws?.credentials?.secretAccessKey)
              && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
                return new EcrRegistry({
                    promise: Promise,
                    credentials: this.inputs.aws?.credentials,
                })
            }
            throw new Error('Registry credentials for ECR not set. Add following registry parameters in your workflow to continue:\n - AWS_ACCESS_KEY\n - AWS_SECRET_KEY\n - AWS_REGION\n')
        }
        if (this.checkNotEmpty(this.inputs.generic?.credentials?.username)
          && this.checkNotEmpty(this.inputs.generic?.credentials?.password)
          && this.checkNotEmpty(this.inputs.generic?.request?.host)) {

            _.set(this.inputs, 'generic.ignoreRedirects', true)
            return new StandardRegistry(this.inputs.generic)
        }
        if (this.inputs.dockerConfigPath) {
            return this.getCredentialsFromDockerConfig(image)
        }
        throw new Error('Registry credentials is required parameter. Add one from following registry parameters in your workflow to continue:\n - Docker credentials: DOCKERHUB_USERNAME, DOCKERHUB_PASSWORD\n - GCR credentials: GCR_KEY_FILE_PATH\n - AWS registry credentials: AWS_ACCESS_KEY, AWS_SECRET_KEY, AWS_REGION\n - Standard registry credentials: REGISTRY_USERNAME, REGISTRY_PASSWORD, REGISTRY_DOMAIN\n')
    }

    async createRegistryClient(image) {
        if (this.inputs.dockerConfigPath) {
            return this.getCredentialsFromDockerConfig(image)
        }

        // Clean this up when have time
        if (this.checkNotEmpty(this.inputs.dockerhub?.username)
          && this.checkNotEmpty(this.inputs.dockerhub?.password)) {

            return new DockerhubRegistry(this.inputs.dockerhub)
        }

        if (this.checkNotEmpty(this.inputs.generic?.credentials?.username)
          && this.checkNotEmpty(this.inputs.generic?.credentials?.password)
          && this.checkNotEmpty(this.inputs.generic?.request?.host)) {

            _.set(this.inputs, 'generic.ignoreRedirects', true)
            return new StandardRegistry(this.inputs.generic)
        }

        if (this.checkNotEmpty(this.inputs.aws?.role)
          && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
            return this.createECRUsingSTS(this.inputs.aws?.role, this.inputs.aws?.credentials?.region)
        }

        if (this.inputs.gcr?.keyFilePath) {
            return new GcrRegistry({
                keyfile: fs.readFileSync(this.inputs.gcr.keyFilePath),
                request: { host: this.inputs.gcr.host || 'gcr.io' }
            })
        }
        if (this.checkNotEmpty(this.inputs.gcr?.json)) {
            return new GcrRegistry({
                keyfile: this.inputs.gcr?.json,
                request: { host: this.inputs.gcr?.host || 'gcr.io' }
            })
        }
        if (this.checkNotEmpty(this.inputs.aws?.credentials?.accessKeyId)
          && this.checkNotEmpty(this.inputs.aws?.credentials?.secretAccessKey)
          && this.checkNotEmpty(this.inputs.aws?.credentials?.region)) {
            return new EcrRegistry({
                promise: Promise,
                credentials: this.inputs.aws?.credentials,
            })
        }
        throw new Error('Registry credentials is required parameter. Add one from following registry parameters in your workflow to continue:\n - Docker credentials: DOCKERHUB_USERNAME, DOCKERHUB_PASSWORD\n - GCR credentials: GCR_KEY_FILE_PATH\n - AWS registry credentials: AWS_ACCESS_KEY, AWS_SECRET_KEY, AWS_REGION\n - Standard registry credentials: REGISTRY_USERNAME, REGISTRY_PASSWORD, REGISTRY_DOMAIN')
    }

    async getRegistryClient(image) {
        if (this.inputs.retrieveCredentialsByDomain) {
            return this.createRegistryClientByImage(image)
        }
        return this.createRegistryClient(image)
    }

    /**
     * @param history {[*]}
     * @returns {[{size, created: (string|Boolean), instruction: string, args: string}]}
     */
    getLayersFromHistory(history, layers) {

        let shell = '/bin/sh -c'
        const instructionPrefix = '#(nop)'

        const layersSizes = _.map(layers, l => l.size)

        history = history.reverse().map((row) => {
            row = row || {}

            const CreatedBy = _.get(row, 'created_by', '')
            let instruction, args

            if (!CreatedBy.includes(instructionPrefix)) {
                instruction = CommonInstructions.run
                args        = CreatedBy.replace(shell, '').trim()
            } else {
                const cmd = CreatedBy.slice(CreatedBy.indexOf(instructionPrefix) + instructionPrefix.length).trim()
                const nextSpacePosition = cmd.indexOf(' ')

                instruction = cmd.slice(0, Math.max(0, nextSpacePosition)).trim()
                args        = cmd.slice(nextSpacePosition).trim()

                if (instruction === CommonInstructions.shell) {
                    shell = (args[0] === '[' && args[args.length - 1] === ']') ? (args
                      .slice(1, -1)
                      .replace(/["']/g, '')
                      .split(',')
                      .map(part => {return part.trim()})
                      .join(' ')) : args
                }
            }

            return {
                created: row.created,
                size: row.empty_layer ? 0 : layersSizes.pop(),
                instruction,
                args
            }
        }).reverse()

        return history
    }

    async buildDockerfile() {
        try {
            const { dockerfileContent, dockerfilePath } = this.inputs
            if (dockerfileContent) {
                return Buffer.from(dockerfileContent, 'base64').toString()
            }
            const file = await fsPromise.readFile(dockerfilePath ? dockerfilePath : '/tmp/Dockerfile') // use path or artifact
            return file.toString()
        } catch (error) {
            console.error(`Can't get Dockerfile. ${error.message}`)
            return ''
        }
    }

    getRegistryImageSize(manifest) {
        return manifest.config.size + _.reduce(manifest.layers, (sum, layer) => {
            return sum + layer.size
        }, 0)
    }

    private decodeBase64 = (str) => Buffer.from(str, 'base64').toString()
}
